@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 J.F.Dockes
+/* Copyright (C) 2005-2025 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -27,6 +27,7 @@
 #include "qxtconfirmationmessage.h"
 
 #include "log.h"
+#include "firstmatchline.h"
 #include "fileudi.h"
 #include "execmd.h"
 #include "transcode.h"
@@ -65,7 +66,7 @@ void RclMain::viewUrl()
        inside internfile and have some url magic to indicate the
        appropriate indexer/identification scheme */
     string udi;
-    make_udi(qs2path(qurl.path()), qs2path(qurl.fragment()), udi);
+    fileUdi::make_udi(qs2path(qurl.path()), qs2path(qurl.fragment()), udi);
     
     Rcl::Doc doc;
     Rcl::Doc idxdoc; // idxdoc.idxi == 0 -> works with base index only
@@ -96,7 +97,7 @@ void RclMain::viewUrl()
         // prevent the temp file deletion completely, leaving it
         // around forever. Better to let the user save a copy if he
         // wants I think.
-        sleep(60);
+        millisleep(60*1000);
         fileExit();
     }
 }
@@ -150,7 +151,7 @@ void RclMain::openWith(Rcl::Doc doc, string cmdspec)
     subs["U"] = beginswith(url, cstr_fileu) ? path_pcencode(url) : url;
     subs["u"] = url;
 
-    execViewer(subs, false, execname, lcmd, cmdspec, doc);
+    execViewer(subs, true, execname, lcmd, cmdspec, doc);
 }
 
 static bool pagenumNeeded(const std::string& cmd)
@@ -163,10 +164,114 @@ static bool linenumNeeded(const std::string& cmd)
 }
 static bool termNeeded(const std::string& cmd)
 {
-    return cmd.find("%s") != std::string::npos;
+    return cmd.find("%s") != std::string::npos || cmd.find("%S") != std::string::npos;
+}
+static bool jsonNeeded(const std::string& cmd)
+{
+    return cmd.find("%j") != std::string::npos;
 }
 
-void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int linenum)
+template <class T> static std::string json_strings(const T& values)
+{
+    std::string jsondata{'['};
+    for (const auto& s: values) {
+        jsondata += json_string(s) + ',';
+    }
+    if (jsondata.back() == ',')
+        jsondata.pop_back();
+    jsondata += ']';
+    return jsondata;
+}
+
+template <class T> static std::string json_stringdict(const T& values)
+{
+    std::string jsondata{'{'};
+    for (const auto& [k,v]: values) {
+        jsondata += json_string(k) + ": " + json_string(v) + ',';
+    }
+    if (jsondata.back() == ',')
+        jsondata.pop_back();
+    jsondata += '}';
+    return jsondata;
+}
+
+static std::string json_stringVV(const vector<vector<string>>& values)
+{
+    std::string jsondata{'['};
+    for (const auto& grp : values) {
+        jsondata += json_strings(grp) + ',';
+    }
+    if (jsondata.back() == ',')
+        jsondata.pop_back();
+    jsondata += "]";
+    return jsondata;
+}
+
+static std::string jsonData(std::shared_ptr<DocSequence> source, Rcl::Doc& doc)
+{
+    vector<vector<string>> docterms;
+    source->getDocTerms(doc, docterms);
+    HighlightData hldata;
+    source->getTerms(hldata);
+
+    std::string jsondata{"{"};
+    
+    jsondata += "\"qterms\": ";
+    jsondata += json_stringVV(docterms);
+
+    jsondata += ", \"hldata\": {";
+
+    jsondata += "\"uterms\": ";
+    jsondata += json_strings(hldata.uterms);
+    jsondata += ", \"terms\": ";
+    jsondata += json_stringdict(hldata.terms);
+    jsondata += ", \"ugroups\": ";
+    jsondata += json_stringVV(hldata.ugroups);
+
+    jsondata += ", \"termgroups\": [";
+    for (const auto& tg : hldata.index_term_groups) {
+        jsondata += "{\"kind\": ";
+        switch (tg.kind) {
+        case HighlightData::TermGroup::TGK_TERM:
+            jsondata += "\"term\",";
+            jsondata += "\"group\": ";
+            jsondata += json_string(tg.term);
+            break;
+        case HighlightData::TermGroup::TGK_NEAR:
+            jsondata += "\"near\",";
+            jsondata += "\"group\": ";
+            jsondata += json_stringVV(tg.orgroups);
+            jsondata += ", \"slack\": ";
+            jsondata += std::to_string(tg.slack);
+            break;
+        case HighlightData::TermGroup::TGK_PHRASE:
+            jsondata += "\"phrase\",";
+            jsondata += "\"group\": ";
+            jsondata += json_stringVV(tg.orgroups);
+            jsondata += ", \"slack\": ";
+            jsondata += std::to_string(tg.slack);
+            break;
+        }
+        jsondata += "},";
+    }
+    if (jsondata.back() == ',')
+        jsondata.pop_back();
+    jsondata += "]";
+
+    jsondata += ", \"spellexpands\": ";
+    jsondata += json_strings(hldata.spellexpands);
+
+    jsondata += "}";
+
+    jsondata += "}";
+    //std::cerr << "JSONDATA:--" << jsondata << "--\n";
+    return jsondata;
+}
+
+// Only the Snippets viewer sets the qterm parameter. Else, if needed, we ask for it from the query
+// through getFirstMatchpage()
+void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int linenum,
+                                bool enterHistory)
 {
     std::string term = qs2utf8s(qterm);
     string apptag;
@@ -188,15 +293,8 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
     ConfSimple viewerattrs;
     string cmd;
     theconfig->valueSplitAttributes(cmdplusattr, cmd, viewerattrs);
-    bool ignoreipath = false;
-    int execwflags = 0;
-    if (viewerattrs.get("ignoreipath", cmdplusattr))
-        ignoreipath = stringToBool(cmdplusattr);
-    if (viewerattrs.get("maximize", cmdplusattr)) {
-        if (stringToBool(cmdplusattr)) {
-            execwflags |= ExecCmd::EXF_MAXIMIZED;
-        }
-    }
+    bool ignoreipath = viewerattrs.getBool("ignoreipath", false);
+    int execwflags = viewerattrs.getBool("maximize", false) ? ExecCmd::EXF_MAXIMIZED : 0;
     
     // Split the command line
     vector<string> lcmd;
@@ -315,9 +413,6 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
         }
     }
 
-    // Can't remember what enterHistory was actually for. Set it to
-    // true always for now
-    bool enterHistory = true;
     bool istempfile = false;
     
     LOGDEB("StartNativeViewer: groksipath " << groksipath << " wantsf " <<
@@ -338,7 +433,6 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
                                  tr("Cannot extract document or create temporary file"));
             return;
         }
-        enterHistory = true;
         istempfile = true;
         rememberTempFile(temp);
         fn = temp.filename();
@@ -379,20 +473,59 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
         prefs.showTempFileWarning = settings.value("/Recoll/prefs/showTempFileWarning").toInt();
     }
 
-    // If we are not called with a page number (which would happen for a call from the snippets
-    // window), see if we can compute a page number anyway. 
-    if (m_source && pagenum == -1 && (pagenumNeeded(cmd) || termNeeded(cmd)|| linenumNeeded(cmd))) {
+    // If we are not called with a page number and/or term (which would happen for a call from the
+    // snippets window), see if we can compute a page number + term anyway.
+    if (m_source &&
+        ((pagenum == -1 && pagenumNeeded(cmd)) || (term.empty() && termNeeded(cmd)))) {
         pagenum = m_source->getFirstMatchPage(doc, term);
-        if (pagenum == -1)
-            pagenum = 1;
     }
 
+    // If a search/find parameter is needed by the command, and if the search was a simple phrase
+    // (expanded to a simple sequence of single terms), possibly generate a multiword
+    // parameter. This works find with evince, which ignores newlines and capital/diacritics when
+    // searching (but not multiple spaces...). Not sure how this would work with other commands, so
+    // it has its own substitution character. This just handles a very specific but common case of a
+    // simple phrase with no term transformations or expansions finding a match, and where users are
+    // wondering why we don't tell the command to search for it.
+    std::string findparam;
+    if (termNeeded(cmd)) {
+        HighlightData hld;
+        m_source->getTerms(hld);
+        LOGDEB1(hld.toString() << '\n');
+        const auto& itg(hld.index_term_groups);
+        if (itg.size() == 1 && itg[0].kind == HighlightData::TermGroup::TGK_PHRASE &&
+            itg[0].slack == 0) {
+            for (const auto& grp : itg[0].orgroups) {
+                if (grp.size() != 1) {
+                    findparam.clear();
+                    break;
+                }
+                findparam += grp[0] + " ";
+            }
+            trimstring(findparam);
+            LOGDEB0("startNativeViewer: findparam [ " << findparam << "]\n");
+        }
+        if (findparam.empty())
+            findparam = term;
+    }
+
+    // Experimental and undocumented at the moment: query match terms (qualityTerms) and
+    // full hldata as json.
+    string jsondata;
+    if (jsonNeeded(cmd))
+        jsondata = jsonData(m_source, doc);
+
+    if (pagenum < 0)
+        pagenum = 1;
+
     if (linenum < 1 && m_source && !term.empty() && linenumNeeded(cmd)) {
         if (doc.text.empty()) {
             rcldb->getDocRawText(doc);
         }
-        linenum = m_source->getFirstMatchLine(doc, term);
+        linenum = getFirstMatchLine(doc, term);
     }
+    if (linenum < 0)
+        linenum = 1;
 
     // Substitute %xx inside arguments
     string efftime;
@@ -401,8 +534,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
     } else {
         efftime = "0";
     }
-    // Try to keep the letters used more or less consistent with the reslist
-    // paragraph format.
+
     map<string, string> subs;
     subs["D"] = efftime;
 #ifdef _WIN32
@@ -411,9 +543,11 @@ void RclMain::startNativeViewer(Rcl::Doc doc, int pagenum, QString qterm, int li
     subs["f"] = fn;
     subs["F"] = fn;
     subs["i"] = FileInterner::getLastIpathElt(doc.ipath);
-    subs["l"] = ulltodecstr(linenum);
+    subs["j"] = jsondata;
+    subs["l"] = std::to_string(linenum);
     subs["M"] = doc.mimetype;
-    subs["p"] = ulltodecstr(pagenum);
+    subs["p"] = std::to_string(pagenum);
+    subs["S"] = findparam;
     subs["s"] = term;
     // Our file:// URLs are actually raw paths. Others should be proper URLs. Only encode file://..
     subs["U"] = beginswith(url, cstr_fileu) ? path_pcencode(url) : url;
@@ -451,14 +585,14 @@ void RclMain::execViewer(
         prcmd = ncmd;
 #else
         string fcharset = theconfig->getDefCharset(true);
-        transcode(ncmd, prcmd, fcharset, "UTF-8");
+        transcode(ncmd, prcmd, fcharset, cstr_utf8);
 #endif
         QString msg = tr("Executing: [") + u8s2qs(prcmd) + "]";
         stb->showMessage(msg, 10000);
     }
 
     if (enterHistory)
-        historyEnterDoc(rcldb.get(), g_dynconf, doc);
+        historyEnterDoc(rcldb, g_dynconf, doc);
     
     // Do the zeitgeist thing
     zg_send_event(ZGSEND_OPEN, doc);
@@ -469,11 +603,6 @@ void RclMain::execViewer(
     ecmd->startExec(execpath, lcmd, false, false);
 }
 
-void RclMain::startManual()
-{
-    startManual(string());
-}
-
 void RclMain::startManual(const string& index)
 {
     string docdir = path_cat(theconfig->getDatadir(), "doc");
@@ -485,28 +614,32 @@ void RclMain::startManual(const string& index)
     webhelp = path_cat(webhelp, "index.html");
     bool has_wh = path_exists(webhelp);
     
-    LOGDEB("RclMain::startManual: help index is " <<
-           (index.empty() ? "(null)" : index) << "\n");
-    bool indexempty = index.empty();
+    LOGDEB("RclMain::startManual: help index is " << (index.empty() ? "(null)" : index) << "\n");
 
-#ifdef _WIN32
-    // On Windows I could not find any way to pass the fragment through
-    // rclstartw (tried to set text/html as exception with rclstartw %u).
-    // So always start the webhelp
-    indexempty = true;
-#endif
-    
-    if (!indexempty) {
+#ifndef _WIN32
+    // On Windows I could not find any way to pass the fragment through rclstartw (tried to set
+    // text/html as exception with rclstartw %u).
+    if (!index.empty()) {
         usermanual += "#";
         usermanual += index;
     }
+#endif
+    
     Rcl::Doc doc;
-    if (has_wh && indexempty) {
+    if (has_wh && index.empty()) {
         doc.url = path_pathtofileurl(webhelp);
     } else {
         doc.url = path_pathtofileurl(usermanual);
     }
     doc.mimetype = "text/html";
-    doc.addmeta(Rcl::Doc::keyapptg, "rclman");
-    startNativeViewer(doc);
+    doc.meta[Rcl::Doc::keyapptg] = "rclman";
+    startNativeViewer(doc, -1, QString(), -1, false);
+}
+
+void RclMain::startOnlineManual()
+{
+    Rcl::Doc doc;
+    doc.url = "https://www.recoll.org/usermanual/webhelp/docs/index.html";
+    doc.mimetype = "text/html";
+    startNativeViewer(doc, -1, QString(), -1, false);
 }
