@@ -303,7 +303,7 @@ int RecollModel::rowCount(const QModelIndex&) const
 int RecollModel::columnCount(const QModelIndex&) const
 {
     LOGDEB2("RecollModel::columnCount\n");
-    return m_fields.size();
+    return static_cast<int>(m_fields.size());
 }
 
 void RecollModel::readDocSource()
@@ -430,7 +430,7 @@ QVariant RecollModel::data(const QModelIndex& index, int role) const
     if (!colname.compare("url")) {
         int ecnt;
         string data1;
-        if (!transcode(data, data1, "UTF-8", "UTF-8", &ecnt) || ecnt > 0) {
+        if (!transcode(data, data1, cstr_utf8, cstr_utf8, &ecnt) || ecnt > 0) {
             data = path_pcencode(data, 7);
         }
     }
@@ -445,32 +445,30 @@ void RecollModel::saveAsCSV(std::fstream& fp)
 {
     if (!m_source)
         return;
-
+    m_source->setqquantum(30000);
+    
     int cols = columnCount();
-    int rows = rowCount();
     vector<string> tokens;
 
     for (int col = 0; col < cols; col++) {
-        QString qs = headerData(col, Qt::Horizontal,Qt::DisplayRole).toString();
-        tokens.push_back((const char *)qs.toUtf8());
+        QString qs = headerData(col, Qt::Horizontal, Qt::DisplayRole).toString();
+        tokens.push_back(qs2utf8s(qs));
     }
-    string csv;
-    stringsToCSV(tokens, csv);
+    auto csv = stringsToCSV(tokens);
     fp << csv << "\n";
-    tokens.clear();
 
-    for (int row = 0; row < rows; row++) {
+    for (int row = 0;; row++) {
         Rcl::Doc doc;
         if (!m_source->getDoc(row, doc)) {
-            continue;
+            break;
         }
         for (int col = 0; col < cols; col++) {
-            tokens.push_back(m_getters[col](m_fields[col], doc));
+            tokens[col] = m_getters[col](m_fields[col], doc);
         }
-        stringsToCSV(tokens, csv);
+        csv = stringsToCSV(tokens);
         fp << csv << "\n";
-        tokens.clear();
     }
+    m_source->setqquantum(100);
 }
 
 // This gets called when the column headers are clicked
@@ -484,15 +482,12 @@ void RecollModel::sort(int column, Qt::SortOrder order)
     DocSeqSortSpec spec;
     if (column >= 0 && column < int(m_fields.size())) {
         spec.field = m_fields[column];
-        if (!stringlowercmp("relevancyrating", spec.field) &&
-            order != Qt::AscendingOrder) {
-            QMessageBox::warning(0, "Recoll",
-                                 tr("Can't sort by inverse relevance"));
+        if (!stringlowercmp("relevancyrating", spec.field) && order != Qt::AscendingOrder) {
+            QMessageBox::warning(0, "Recoll", tr("Can't sort by inverse relevance"));
             QTimer::singleShot(0, m_table, SLOT(resetSort()));
             return;
         }
-        if (!stringlowercmp("date", spec.field) ||
-            !stringlowercmp("datetime", spec.field))
+        if (!stringlowercmp("date", spec.field) || !stringlowercmp("datetime", spec.field))
             spec.field = "mtime";
         spec.desc = (order != Qt::AscendingOrder);
     }
@@ -609,10 +604,27 @@ void ResTable::setDefRowHeight()
 
 void ResTable::init(QStringList _ifields)
 {
-    QSettings settings;
     QStringList restableFields;
-    if (_ifields.empty()) {
-        restableFields = settings.value(settingskey_fieldlist).toStringList();
+    QString qw;
+    QVariant splittersizes;
+    
+    if (!_ifields.empty()) {
+        restableFields = _ifields;
+    } else {
+        bool locset;
+        if (theconfig->getConfParam("idxlocalguisettings", &locset) && locset) {
+            QSettings settings(
+                u8s2qs(path_cat(theconfig->getConfDir(), "recollgui.ini")), QSettings::IniFormat);
+            restableFields = settings.value(settingskey_fieldlist).toStringList();
+            qw = settings.value(settingskey_fieldwiths).toString();
+            splittersizes = settings.value(settingskey_splittersizes);            
+        }
+        if (restableFields.empty()) {
+            QSettings settings;
+            restableFields = settings.value(settingskey_fieldlist).toStringList();
+            qw = settings.value(settingskey_fieldwiths).toString();
+            splittersizes = settings.value(settingskey_splittersizes);
+        }
         if (restableFields.empty()) {
             restableFields.push_back("date");
             restableFields.push_back("title");
@@ -620,9 +632,8 @@ void ResTable::init(QStringList _ifields)
             restableFields.push_back("author");
             restableFields.push_back("url");
         }
-    } else {
-        restableFields = _ifields;
     }
+
     if (!(m_model = new RecollModel(restableFields, this)))
         return;
     tableView->setModel(m_model);
@@ -646,7 +657,6 @@ void ResTable::init(QStringList _ifields)
     QHeaderView *header = tableView->horizontalHeader();
     if (header) {
         if (_ifields.empty()) {
-            QString qw = settings.value(settingskey_fieldwiths).toString();
             vector<string> vw;
             stringToStrings(qs2utf8s(qw), vw);
             vector<int> restableColWidths;
@@ -694,12 +704,12 @@ void ResTable::init(QStringList _ifields)
     m_detail->setOpenLinks(false);
     m_detail->init();
     // signals and slots connections
-    connect(m_detail, SIGNAL(anchorClicked(const QUrl&)), this, SLOT(linkWasClicked(const QUrl&)));
+    connect(m_detail, SIGNAL(anchorClicked(const QUrl&)), this, SLOT(onLinkClicked(const QUrl&)));
     splitter->addWidget(m_detail);
     splitter->setOrientation(Qt::Vertical);
-    QVariant saved = settings.value(settingskey_splittersizes);
-    if (saved != QVariant()) {
-        splitter->restoreState(saved.toByteArray());
+
+    if (splittersizes != QVariant()) {
+        splitter->restoreState(splittersizes.toByteArray());
     } else {
         QList<int> sizes;
         sizes << 355 << 125;
@@ -895,8 +905,15 @@ void ResTable::saveColState()
 {
     if (!m_ismainres)
         return;
-    QSettings settings;
-    settings.setValue(settingskey_splittersizes, splitter->saveState());
+    QSettings gsettings;
+    QSettings lsettings(
+        u8s2qs(path_cat(theconfig->getConfDir(), "recollgui.ini")), QSettings::IniFormat);
+    QSettings *settings = &gsettings;
+    bool locset;
+    if (theconfig->getConfParam("idxlocalguisettings", &locset) && locset) {
+        settings = &lsettings;
+    }
+    settings->setValue(settingskey_splittersizes, splitter->saveState());
 
     QHeaderView *header = tableView->horizontalHeader();
     const vector<string>& vf = m_model->getFields();
@@ -918,8 +935,8 @@ void ResTable::saveColState()
         newfields.push_back(u8s2qs(vf[li]));
         newwidths += QString().setNum(header->sectionSize(li)) + QString(" ");
     }
-    settings.setValue(settingskey_fieldlist, newfields);
-    settings.setValue(settingskey_fieldwiths, newwidths);
+    settings->setValue(settingskey_fieldlist, newfields);
+    settings->setValue(settingskey_fieldwiths, newwidths);
 }
 
 void ResTable::onTableView_currentChanged(const QModelIndex& index)
@@ -1026,20 +1043,24 @@ void ResTable::resetSource()
     readDocSource();
 }
 
+
 void ResTable::saveAsCSV()
 {
+    static const QString csvprevdirkey("/Recoll/prefs/restableAsCSVPDir");;
     LOGDEB("ResTable::saveAsCSV\n");
     if (!m_model)
         return;
-    QString s = QFileDialog::getSaveFileName(
-        this, tr("Save table to CSV file"), path2qs(path_home()));
+    QSettings settings;
+    QString s = QFileDialog::getSaveFileName(this, tr("Save table to CSV file"),
+                                             settings.value(csvprevdirkey).toString());
     if (s.isEmpty())
         return;
+    QDir cdir;
+    settings.setValue(csvprevdirkey, cdir.absoluteFilePath(s));
     std::string tofile = qs2path(s);
     std::fstream fp;
-    if (!path_streamopen(tofile, std::ios::out|std::ios::trunc,fp)) {
-        QMessageBox::warning(0, "Recoll",
-                             tr("Can't open/create file: ") + s);
+    if (!path_streamopen(tofile, std::ios::out|std::ios::trunc, fp)) {
+        QMessageBox::warning(0, "Recoll", tr("Can't open/create file: ") + s);
         return;
     }
     m_model->saveAsCSV(fp);
@@ -1049,8 +1070,7 @@ void ResTable::saveAsCSV()
 // This is called when the sort order is changed from another widget
 void ResTable::onSortDataChanged(DocSeqSortSpec spec)
 {
-    LOGDEB("ResTable::onSortDataChanged: [" << spec.field << "] desc " <<
-           spec.desc << "\n");
+    LOGDEB("ResTable::onSortDataChanged: [" << spec.field << "] desc " << spec.desc << "\n");
     QHeaderView *header = tableView->horizontalHeader();
     if (!header || !m_model)
         return;
@@ -1099,22 +1119,23 @@ void ResTable::readDocSource(bool resetPos)
     m_detaildocnum = -1;
 }
 
-void ResTable::linkWasClicked(const QUrl &qurl)
+void ResTable::onLinkClicked(const QUrl &qurl)
 {
     if (m_detaildocnum < 0) {
         return;
     }
     auto strurl = pc_decode(qs2utf8s(qurl.toString()));
-    LOGDEB("ResTable::linkWasClicked: [" << strurl << "]\n");
-
-    auto docseqnum = atoi(strurl.c_str()+1) -1;
+    LOGDEB("ResTable::onLinkClicked: [" << strurl << "]\n");
+    auto [what, docseqnum, origorscript, replacement] = internal_link(strurl);
+    if (what == 0) {
+        return;
+    }
     if (m_detaildocnum != docseqnum) {
         //? Really we should abort...
-        LOGERR("ResTable::linkWasClicked: m_detaildocnum != docseqnum !\n");
+        LOGERR("ResTable::onLinkClicked: m_detaildocnum != docseqnum !\n");
         return;
     }
     
-    int what = strurl[0];
     switch (what) {
         // Open abstract/snippets window
     case 'A':
@@ -1171,8 +1192,8 @@ void ResTable::linkWasClicked(const QUrl &qurl)
     break;
 
     default:
-        LOGERR("ResTable::linkWasClicked: bad link [" << strurl << "]\n");
-        break;// ??
+        LOGERR("ResTable::onLinkClicked: bad link [" << strurl << "]\n");
+        break;
     }
 }
 
